@@ -27,7 +27,7 @@ coords.smooth.roc <- function(smooth.roc, x, input=c("specificity", "sensitivity
   # match input 
   input <- match.arg(input)
   # match return
-  valid.ret.args <- c("specificity", "sensitivity", "accuracy", "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity", "1-sensitivity", "1-npv", "1-ppv")
+  valid.ret.args <- c("specificity", "sensitivity", "accuracy", "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity", "1-sensitivity", "1-accuracy", "1-npv", "1-ppv")
   ret <- replace(ret, ret=="npe", "1-npv")
   ret <- replace(ret, ret=="ppe", "1-ppv")
   ret <- match.arg(ret, valid.ret.args, several.ok=TRUE)
@@ -58,14 +58,14 @@ coords.smooth.roc <- function(smooth.roc, x, input=c("specificity", "sensitivity
       sp <- smooth.roc$specificities[optim.crit==max(optim.crit)]
     }
     else {
-      if (attr(roc$auc, "partial.auc.focus") == "sensitivity") {
-        optim.crit <- (optim.crit)[roc$se <= partial.auc[1] & roc$se >= partial.auc[2]]
+      if (attr(smooth.roc$auc, "partial.auc.focus") == "sensitivity") {
+        optim.crit <- (optim.crit)[smooth.roc$se <= partial.auc[1] & smooth.roc$se >= partial.auc[2]]
         se <- smooth.roc$sensitivities[smooth.roc$sensitivities <= partial.auc[1] & smooth.roc$sensitivities >= partial.auc[2]][optim.crit==max(optim.crit)]
-        sp <- smooth.roc$specificities[smooth.roc$specificities <= partial.auc[1] & smooth.roc$specificities >= partial.auc[2]][optim.crit==max(optim.crit)]
+        sp <- smooth.roc$specificities[smooth.roc$sensitivities <= partial.auc[1] & smooth.roc$sensitivities >= partial.auc[2]][optim.crit==max(optim.crit)]
       }
       else {
-        optim.crit <- (optim.crit)[roc$sp <= partial.auc[1] & roc$sp >= partial.auc[2]]
-        se <- smooth.roc$sensitivities[smooth.roc$sensitivities <= partial.auc[1] & smooth.roc$sensitivities >= partial.auc[2]][optim.crit==max(optim.crit)]
+        optim.crit <- (optim.crit)[smooth.roc$sp <= partial.auc[1] & smooth.roc$sp >= partial.auc[2]]
+        se <- smooth.roc$sensitivities[smooth.roc$specificities <= partial.auc[1] & smooth.roc$specificities >= partial.auc[2]][optim.crit==max(optim.crit)]
         sp <- smooth.roc$specificities[smooth.roc$specificities <= partial.auc[1] & smooth.roc$specificities >= partial.auc[2]][optim.crit==max(optim.crit)]
       }
     }
@@ -93,7 +93,7 @@ coords.smooth.roc <- function(smooth.roc, x, input=c("specificity", "sensitivity
     accuracy <- (tp + tn) / (tp + tn + fp + fn)
     if (length(se) == 1) {
       if (as.list) {
-        list <- list(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)[ret]
+        list <- list(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-accuracy"=substr.percent-accuracy, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)[ret]
         if (drop == FALSE) {
           list <- list(list)
           names(list) <- x
@@ -101,7 +101,7 @@ coords.smooth.roc <- function(smooth.roc, x, input=c("specificity", "sensitivity
         return(list)
       }
       else {
-        res <- c(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)[ret]
+        res <- c(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-accuracy"=substr.percent-accuracy, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)[ret]
         if (drop == FALSE) {
         }
         return(res)
@@ -109,12 +109,12 @@ coords.smooth.roc <- function(smooth.roc, x, input=c("specificity", "sensitivity
     }
     else if (length(se) > 1) {
       if (as.list) {
-        co <- apply(rbind(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv), 2, as.list)
+        co <- apply(rbind(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-accuracy"=substr.percent-accuracy, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv), 2, as.list)
         names(co) <- rep("best", length(co))
         return(co)
       }
       else {
-        co <- rbind(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)
+        co <- rbind(sensitivity=se, specificity=sp, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-accuracy"=substr.percent-accuracy, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)
         colnames(co) <- rep(x, dim(co)[2])
         return(co)
       }
@@ -133,11 +133,7 @@ coords.roc <- function(roc, x, input=c("threshold", "specificity", "sensitivity"
   # match input 
   input <- match.arg(input)
   # match return
-  valid.ret.args <- c("threshold", "specificity", "sensitivity", "accuracy", "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity", "1-sensitivity", "1-npv", "1-ppv")
-  ret <- replace(ret, ret=="t", "threshold")
-  ret <- replace(ret, ret=="npe", "1-npv")
-  ret <- replace(ret, ret=="ppe", "1-ppv")
-  ret <- match.arg(ret, valid.ret.args, several.ok=TRUE)
+  ret <- roc.utils.match.coords.ret.args(ret)
   # make sure the sort of roc is correct
   roc <- sort(roc)
 
@@ -314,7 +310,7 @@ coords.roc <- function(roc, x, input=c("threshold", "specificity", "sensitivity"
     }
     accuracy <- (tp + tn) / (tp + tn + fp + fn)
     if (as.list) {
-      list <- list(threshold=res[1], specificity=sp, sensitivity=se, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)
+      list <- list(threshold=res[1], specificity=sp, sensitivity=se, accuracy=accuracy, tn=tn, tp=tp, fn=fn, fp=fp, npv=npv, ppv=ppv, "1-specificity"=substr.percent-sp, "1-sensitivity"=substr.percent-se, "1-accuracy"=substr.percent-accuracy, "1-npv"=substr.percent-npv, "1-ppv"=substr.percent-ppv)
       list <- list[ret]
       if (drop == FALSE) {
         list <- list(list)
@@ -324,8 +320,8 @@ coords.roc <- function(roc, x, input=c("threshold", "specificity", "sensitivity"
     }
     else {
       res <- as.matrix(res)
-      res <- rbind(res, accuracy, tn, tp, fn, fp, npv, ppv, substr.percent-sp, substr.percent-se, substr.percent-npv, substr.percent-ppv)
-      rownames(res) <- c("threshold", "specificity", "sensitivity", "accuracy", "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity", "1-sensitivity", "1-npv", "1-ppv")
+      res <- rbind(res, accuracy, tn, tp, fn, fp, npv, ppv, substr.percent-sp, substr.percent-se, substr.percent-accuracy, substr.percent-npv, substr.percent-ppv)
+      rownames(res) <- c("threshold", "specificity", "sensitivity", "accuracy", "tn", "tp", "fn", "fp", "npv", "ppv", "1-specificity", "1-sensitivity", "1-accuracy", "1-npv", "1-ppv")
       colnames(res) <- x
       return(res[ret,, drop=drop])
     }
