@@ -40,7 +40,7 @@ cov.smooth.roc <- function(roc1, roc2, ...) {
 }
 
 cov.roc <- function(roc1, roc2,
-                         method=c("delong", "bootstrap"),
+                         method=c("delong", "bootstrap", "obuchowski"),
                          reuse.auc=TRUE,
                          boot.n=2000, boot.stratified=TRUE, boot.return=FALSE,
                          progress=getOption("pROCProgress")$name,
@@ -152,6 +152,19 @@ cov.roc <- function(roc1, roc2,
       if (roc1$direction != roc2$direction)
         warning("DeLong should not be applied to ROC curves with a different direction.")
     }
+    else if (method == "obuchowski") {
+      if (smoothing.args$roc1$smooth || smoothing.args$roc2$smooth) {
+        warning("Using Obuchowski for smoothed ROCs is not supported. Using bootstrap instead.")
+        method <- "bootstrap"
+      }
+      if ((has.partial.auc(roc1) && attr(roc1$auc, "partial.auc.focus") == "sensitivity")
+          || (has.partial.auc(roc2) && attr(roc2$auc, "partial.auc.focus") == "sensitivity")) {
+        warning("Using Obuchowski for partial AUC on sensitivity region is not supported. Using bootstrap instead.")
+        method <- "bootstrap"
+      }
+      if (roc1$direction != roc2$direction)
+        warning("Obuchowski should not be applied to ROC curves with a different direction.")
+    }
   }
   
   if (method == "delong") {
@@ -170,6 +183,14 @@ cov.roc <- function(roc1, roc2,
       cov <- cov * (100^2)
     }
   }
+  
+  else if (method == "obuchowski") {
+    cov <- cov.roc.obuchowski(roc1, roc2) / length(roc1$cases)
+
+    if (roc1$percent) {
+      cov <- cov * (100^2)
+    }
+  }
   else { # method == "bootstrap"
     # Check if called with density.cases or density.controls
     if (is.null(smoothing.args) || is.numeric(smoothing.args$density.cases) || is.numeric(smoothing.args$density.controls))
